from utils import read_csv, save_csv, remove_duplicates, CONFIG, join_dfs
import lxml.etree as et
import operator
import pandas as pd
from numpy import nan
import time

config = read_csv(CONFIG)
countries = ['se'] #['se', 'dk', 'fi']

def xml_to_csv():
    for country in countries:
        counter = 0
        for warehouse in config:
            list_to_save = []
            xml = f'download/{warehouse["warehouse"]}_{country}.xml'
            print('preparing csv file from',xml)
            found = 0
            not_found = 0
            for event, elem in et.iterparse(xml):
                ean = 0
                if elem.tag == "o":
                    try:
                        for attr in elem.find('attrs').findall('a'):
                            if attr.attrib['name'] == 'EAN':
                                ean = attr.text.strip().replace(',00','')
                        if int(elem.attrib['stock']) > 0:
                            found += 1
                            ean = int(ean)
                            list_to_save.append({
                                'sku_'+country: elem.attrib['id'],
                                'ean_'+country: ean,
                                'price_feed_'+country: float(elem.attrib['price']),
                                'stock_feed_'+country: elem.attrib['stock'],
                                'shipping_'+country: warehouse['shipping_'+country],
                                'deliveryTime_min_'+country: warehouse['deliveryTime_min'],
                                'deliveryTime_max_'+country: warehouse['deliveryTime_max'],
                            })
                    except Exception as e:
                        #if not isinstance(e, AttributeError):
                            #print(e)
                            #time.sleep(1)
                        not_found += 1
            print("found ", found)
            print("not found ", not_found)
            print(f'{xml} length of number of products from feed {len(list_to_save)}')
            newlist = sorted(list_to_save, key=operator.itemgetter('ean_'+country))
            newlist = remove_duplicates(newlist, on = 'ean_'+country)
            print(f'{xml} length of number of products from feed sorted {len(newlist)}')
            save_csv(f'download/{country}_eans{counter}.csv', newlist)
            counter += 1
        counter = 0

def concat_feeds():
    for country in countries:
        counter = 0
        for warehouse in config:
            df1 = None
            for counter in range(len(config)-1):
                if df1 is None:
                    df1 = pd.read_csv(f'download/{country}_eans{counter}.csv', sep = ';')
                df2 = pd.read_csv(f'download/{country}_eans{counter+1}.csv', sep = ';')
                df1 = pd.concat([df1, df2], axis=0)
            df1['ean_'+country] = df1['ean_'+country].astype('int64')
            df1.drop_duplicates(subset=['ean_'+country], keep='first', inplace=True)
            df1.to_csv(f'download/{country}_eans.csv', sep=';', index=False)
            
def merge_feeds():
    joined_products = read_csv(f'download/nonvida_skus.csv', to_dict = False)
    for country in countries:
        products_from_feeds = read_csv(f'download/{country}_eans.csv', to_dict = False)
        joined_products = join_dfs(joined_products, products_from_feeds, left_on = 'sku', right_on = 'sku_'+country, how='left', indicator = 'indicator_'+country)
    joined_products.replace(nan, 0, inplace=True)
    joined_products = joined_products.astype({'ean_se': 'int64', 'ean_dk': 'int64', 'ean_fi': 'int64', 'deliveryTime_min_se': int, 'deliveryTime_max_se': int, 'deliveryTime_min_dk': int, 'deliveryTime_max_dk': int, 'deliveryTime_min_fi': int, 'deliveryTime_max_fi': int,})
    joined_products.to_csv(f'download/all_nonvida.csv', sep=';', index=False)

def execute():
    xml_to_csv()
    concat_feeds()
    merge_feeds()
    
if __name__ == "__main__":
    execute()

